@@ -3,7 +3,7 @@ use Object::Pad ':experimental(init_expr)';
 
 package OpenTelemetry::Exporter::OTLP::Encoder::Protobuf;
 
-our $VERSION = '0.016';
+our $VERSION = '0.017';
 
 class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
     :isa(OpenTelemetry::Exporter::OTLP::Encoder::JSON) {
@@ -16,8 +16,15 @@ class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
     method content_type () { 'application/x-protobuf' }
 
     method serialise ($data) {
-        OpenTelemetry::Proto::Collector::Trace::V1::ExportTraceServiceRequest
-            ->new_and_check({ resource_spans => $data->{resourceSpans} })->encode;
+        return OpenTelemetry::Proto::Collector::Logs::V1::ExportLogsServiceRequest
+            ->new_and_check({ resource_logs => $data->{resourceLogs} })->encode
+            if $data->{resourceLogs};
+
+        return OpenTelemetry::Proto::Collector::Trace::V1::ExportTraceServiceRequest
+            ->new_and_check({ resource_spans => $data->{resourceSpans} })->encode
+            if $data->{resourceSpans};
+
+        die 'Unknown payload type';
     }
 
     method encode_anyvalue ( $v ) {
@@ -33,7 +40,7 @@ class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
         }
 
         # TODO: not strings
-        return { string_value => $v };
+        return { string_value => "$v" };
     }
 
     method encode_resource ( $resource ) {
@@ -43,7 +50,7 @@ class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
         };
     }
 
-    method encode_event ( $event ) {
+    method encode_span_event ( $event ) {
         {
             attributes               => $self->encode_kvlist($event->attributes),
             dropped_attributes_count => $event->dropped_attributes,
@@ -52,7 +59,7 @@ class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
         };
     }
 
-    method encode_link ( $link ) {
+    method encode_span_link ( $link ) {
         {
             attributes               => $self->encode_kvlist($link->attributes),
             dropped_attributes_count => $link->dropped_attributes,
@@ -68,13 +75,13 @@ class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
             dropped_events_count     => $span->dropped_events,
             dropped_links_count      => $span->dropped_links,
             end_time_unix_nano       => int $span->end_timestamp * 1_000_000_000,
-            events                   => [ map $self->encode_event($_), $span->events ],
+            events                   => [ map $self->encode_span_event($_), $span->events ],
             kind                     => $span->kind,
-            links                    => [ map $self->encode_link($_),  $span->links  ],
+            links                    => [ map $self->encode_span_link($_),  $span->links  ],
             name                     => $span->name,
             span_id                  => $span->span_id,
             start_time_unix_nano     => int $span->start_timestamp * 1_000_000_000,
-            status                   => $self->encode_status($span->status),
+            status                   => $self->encode_span_status($span->status),
             trace_id                 => $span->trace_id,
             trace_state              => $span->trace_state->to_string,
         };
@@ -86,6 +93,25 @@ class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
         $data;
     }
 
+    method encode_log ( $log ) {
+        my $data = {
+            attributes               => $self->encode_kvlist($log->attributes),
+            body                     => $self->encode_anyvalue( $log->body ),
+            dropped_attributes_count => $log->dropped_attributes,
+            flags                    => $log->trace_flags->flags,
+            observed_time_unix_nano  => int $log->observed_timestamp * 1_000_000_000,
+            severity_number          => 0+$log->severity_number,
+            severity_text            => $log->severity_text,
+            span_id                  => $log->span_id,
+            trace_id                 => $log->trace_id,
+        };
+
+        my $t = $log->timestamp;
+        $data->{time_unix_nano} = int $t * 1_000_000_000 if defined $t;
+
+        $data;
+    }
+
     method encode_scope ( $scope ) {
         {
             attributes               => $self->encode_kvlist($scope->attributes),
@@ -95,6 +121,13 @@ class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
         };
     }
 
+    method encode_scope_logs ( $scope, $logs ) {
+        {
+            scope       => $self->encode_scope($scope),
+            log_records => [ map $self->encode_log($_), @$logs ],
+        };
+    }
+
     method encode_resource_spans ( $resource, $spans ) {
         my %scopes;
 
@@ -113,4 +146,23 @@ class OpenTelemetry::Exporter::OTLP::Encoder::Protobuf
             schema_url => $resource->schema_url,
         };
     }
+
+    method encode_resource_logs ( $resource, $logs ) {
+        my %scopes;
+
+        for (@$logs) {
+            my $key = refaddr $_->instrumentation_scope;
+
+            $scopes{ $key } //= [ $_->instrumentation_scope, [] ];
+            push @{ $scopes{ $key }[1] }, $_;
+        }
+
+        {
+            resource => $self->encode_resource($resource),
+            scope_logs => [
+                map $self->encode_scope_logs(@$_), values %scopes,
+            ],
+            schema_url => $resource->schema_url,
+        };
+    }
 }
